@@ -2,9 +2,10 @@
 TODO: Modify unittest doc.
 """
 
+from __future__ import annotations
+
 import os
 import sys
-import unittest
 
 from monty.termcolor import (
     cprint,
@@ -75,7 +76,3 @@ class TestFunc:
     def test_stream_has_colors(self):
         # TODO: not a real test. Need to do a proper test.
         stream_has_colours(sys.stdout)
-
-
-if __name__ == "__main__":
-    unittest.main()
